<?php

namespace Drupal\Tests\many_selects\Functional;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\link\LinkItemInterface;
use Drupal\Tests\node\Functional\NodeTestBase;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\system\Functional\Cache\AssertPageCacheContextsAndTagsTrait;
use Drupal\Tests\system\Functional\Menu\AssertBreadcrumbTrait;

/**
 * Testing the integrity of the site after the module was installed.
 *
 * @group many_selects
 */
class ManySelectsAdminTest extends NodeTestBase {

  use AssertBreadcrumbTrait;
  use AssertPageCacheContextsAndTagsTrait;
  use ContentTypeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'link', 'many_selects'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Make sure to complete the normal setup steps first.
    parent::setUp();

    // Create and log in an administrative user.
    $this->drupalLogin($this->rootUser);
  }

  /**
   * Tests that your site is able to create content types.
   */
  public function testBasicNodeWithListField() {
    $bundle = 'restaurant';
    $fieldName = 'field_list_dishes';

    $this->drupalCreateContentType([
      'type' => $bundle,
      'name' => 'Restaurant',
    ]);

    $fieldStorage = FieldStorageConfig::create([
      'field_name' => $fieldName,
      'entity_type' => 'node',
      'type' => 'link',
    ]);

    $fieldStorage->save();

    $field = FieldConfig::create([
      'field_storage' => $fieldStorage,
      'bundle' => $bundle,
      'settings' => [
        'title' => DRUPAL_DISABLED,
        'link_type' => LinkItemInterface::LINK_GENERIC,
      ],
    ]);

    $field->save();

    $article = $this->drupalCreateNode([
      'type' => $bundle,
      'title' => 'Test title',
      'body' => [
        'value' => 'Hello world!',
        'format' => 'plain_text',
      ],
      $fieldName => [
        'title' => 'Link Title',
        'uri' => 'uri',
      ],
    ]);

    $this->assertEquals($article->get($fieldName)->first()->getValue(), [
      'title' => 'Link Title',
      'uri' => 'uri',
      'options' => [],
    ]);

  }

  /**
   * Tests if user can load the front page among other things.
   */
  public function testLoadSite() {
    // Load the front page.
    $this->drupalGet('<front>');

    // Confirm that the site didn't throw a server error or something else.
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("user");
  }

}
